(define-module (ordo host)
  #:use-module (ice-9 match)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-9)
  #:use-module (ordo connection)
  #:export (make-host
            host?
            host-name
            host-connection
            host-tags
            resolve-hosts))

(define-record-type <host>
  (make-host name connection tags)
  host?
  (name host-name)
  (connection host-connection)
  (tags host-tags))

(define (tagged-all? wanted-tags)
  (lambda (h)
    (lset= equal? wanted-tags (lset-intersection equal? (host-tags h) wanted-tags))))

(define (tagged-any? wanted-tags)
  (lambda (h)
    (not (null? (lset-intersection equal? (host-tags h) wanted-tags)))))

(define (named? hostname)
  (lambda (h)
    (string=? (host-name h) hostname)))

(define (resolve-hosts inventory)
  (match-lambda
    ("localhost" (list (or (find (named? "localhost") inventory)
                           (make-host "localhost" (local-connection) '()))))
    ((? string? hostname) (filter (named? hostname) inventory))
    ('all inventory)
    (('every-tag tag . tags) (filter (tagged-all? (cons tag tags)) inventory))
    (('any-tag tag . tags) (filter (tagged-any? (cons tag tags)) inventory))))
