(define-module (ordo play)
  #:use-module (srfi srfi-9)
  #:use-module (srfi srfi-26)
  #:use-module (logging logger)
  #:use-module (ordo connection)
  #:use-module (ordo context)
  #:use-module (ordo task)
  #:use-module (ordo handler)
  #:use-module (ordo context)
  #:use-module (ordo host)
  #:use-module (ordo facts)
  #:export (play
            play?
            play-host
            play-sudo?
            play-sudo-user
            play-sudo-password
            play-vars
            play-tasks
            play-handlers
            play-gather-facts
            run-play))

(define-record-type <play>
  (make-play name host sudo? sudo-user sudo-password vars gather-facts tasks handlers)
  play?
  (name play-name)
  (host play-host)
  (sudo? play-sudo?)
  (sudo-user play-sudo-user)
  (sudo-password play-sudo-password)
  (vars play-vars)
  (tasks play-tasks)
  (handlers play-handlers)
  (gather-facts play-gather-facts))

(define* (play name #:key host (sudo? #f) (sudo-user #f) (sudo-password #f) (vars '()) (gather-facts #t) . more)
  (let ((tasks (filter task? more))
        (handlers (filter handler? more)))
    (make-play name host sudo? sudo-user sudo-password vars gather-facts tasks handlers)))

(define (run-play p)
  (log-msg 'NOTICE "Running play: " (play-name p))
  (let ((hosts ((resolve-hosts (current-inventory)) (play-host p))))
    (if (null? hosts)
        (log-msg 'WARN "No hosts matched: " (play-host p))
        (for-each (lambda (h) (run-host-play p h)) hosts))))

(define (run-host-play p h)
  (log-msg 'NOTICE "Running play: " (play-name p) " on host: " (host-name h))
  (call-with-connection
   (host-connection h)
   (play-sudo? p)
   (play-sudo-user p)
   (play-sudo-password p)
   (lambda (conn)
     (dynamic-wind
       (lambda ()
         (set-current-connection! conn)
         (set-current-host! (host-name h))
         (init-play-vars! (play-vars p)))
       (lambda ()
         (when (play-gather-facts p) (gather-facts))
         (for-each run-task (play-tasks p))
         (for-each run-handler
                   (filter (compose play-triggered? handler-name) (play-handlers p))))
       (lambda ()
         (set-current-connection! #f)
         (set-current-host! #f)
         (reset-play-vars!)
         (reset-play-triggers!))))))
